#include <windows.h>
#include <ddraw.h>
#include "Global.h"
#include <mmsystem.h>
#include <stdio.h>

void GameMain()
{
	if(GetAsyncKeyState(VK_ESCAPE) & 0x8000)
	{
		PostQuitMessage(0);
		return;
	}

	DDBLTFX ddbltfx;

	memset(&ddbltfx, 0, sizeof(ddbltfx));
	ddbltfx.dwSize = sizeof(ddbltfx);
	ddbltfx.dwFillColor = RGBVAL(0, 0, 0);

	// Clear Screen
	if(FAILED(g_pBack->Blt(0, 0, 0, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx)))
	{
		MessageBox(g_hwnd, "Clear Screen Blt Failed", "Error", MB_OK);
		Log("Clear Screen Blt Failed");
		return;
	}

	if(Game.win)
	{
		DisplayText(368, 295, "You Win!", 0x00ffff);
		if(GetAsyncKeyState(VK_RETURN) & 0x8000)
		{
			Game.win = false;
			Ball.Velx = 2;
			Ball.Vely = 3;
			Enemy.x1 = 380;
			Enemy.x2 = 420;
		}
	}
	else if(Game.lose)
	{
		DisplayText(368, 295, "You Lose!", 0x00ffff);
		if(GetAsyncKeyState(VK_RETURN) & 0x8000)
		{
			Game.lose = false;
			Game.Score -= 400;
			if(Game.Score < 0) Game.Score = 0;
			Game.level = 1;
			Ball.Velx = -2;
			Ball.Vely = -3;
			Enemy.x1 = 380;
			Enemy.x2 = 420;
		}
	}
	else if(Game.over)
	{
		DisplayText(368, 295, "Game Over!", 0x00ffff);
		if(GetAsyncKeyState(VK_RETURN) & 0x8000)
		{
			Game.over = false;
			Game.level = 1;
			Game.lives = 3;
			Game.Score = 0;
			Ball.Velx = -2;
			Ball.Vely = -3;
			Enemy.x1 = 380;
			Enemy.x2 = 420;
		}
	}
	else
	{	
		GetInput(); // Check Input
		Collision(); // Collision detection

		static DWORD prevTime = 0;
		if(timeGetTime() >= prevTime)
		{
			Ball.x1 += Ball.Velx;
			Ball.x2 += Ball.Velx;
			Ball.y1 += Ball.Vely;
			Ball.y2 += Ball.Vely;
			prevTime = timeGetTime() + 5;
		}
		
		// Emeny AI
		if(Ball.x1 <= Enemy.x1)
		{
			Enemy.x1 -= 2;
			Enemy.x2 -= 2;
		}
		else if(Ball.x2 >= Enemy.x2)
		{
			Enemy.x1 += 2;
			Enemy.x2 += 2;
		}
	}

	RenderScene(); // RenderScene :)

	char OutputBuffer[70];

	sprintf(OutputBuffer, "Score: %d", Game.Score);
	DisplayText(250, 125, OutputBuffer, 0xffffff);

	memset(OutputBuffer, 0, 70);
	sprintf(OutputBuffer, "Lives: %d", Game.lives);
	DisplayText(110, 125, OutputBuffer, 0xffffff);

	memset(OutputBuffer, 0, 70);
	sprintf(OutputBuffer, "Level: %d", Game.level);
	DisplayText(180, 125, OutputBuffer, 0xffffff);

	memset(OutputBuffer, 0, 70);
	sprintf(OutputBuffer, "ESC to quit, arrow keys to move, Enter on prompt and F5 for debug info.");
	DisplayText(165, 570, OutputBuffer, 0xCCCCCC);

	if(Game.showDebug)
	{
		memset(OutputBuffer, 0, 50);
		sprintf(OutputBuffer, "FPS: %d", FPSCounter());
		DisplayText(1, 150, OutputBuffer, 0xffffff);

		memset(OutputBuffer, 0, 50);
		sprintf(OutputBuffer, "Ball.Velx: %d", Ball.Velx);
		DisplayText(1, 165, OutputBuffer, 0xffffff);

		memset(OutputBuffer, 0, 50);
		sprintf(OutputBuffer, "Ball.Vely: %d", Ball.Vely);
		DisplayText(1, 180, OutputBuffer, 0xffffff);

		memset(OutputBuffer, 0, 50);
		sprintf(OutputBuffer, "Ball.x1: %d", Ball.x1);
		DisplayText(1, 195, OutputBuffer, 0xffffff);

		memset(OutputBuffer, 0, 50);
		sprintf(OutputBuffer, "Ball.y1: %d", Ball.y1);
		DisplayText(1, 210, OutputBuffer, 0xffffff);

		memset(OutputBuffer, 0, 50);
		sprintf(OutputBuffer, "Player.x1: %d", Player.x1);
		DisplayText(1, 225, OutputBuffer, 0xffffff);

		memset(OutputBuffer, 0, 50);
		sprintf(OutputBuffer, "Player.x2: %d", Player.x2);
		DisplayText(1, 240, OutputBuffer, 0xffffff);
	}


	while(FAILED(g_pPrimary->Flip(NULL, DDFLIP_WAIT)));
}

void GetInput()
{
	static DWORD compareTimeMovement = 0;
	static DWORD lastTimeShow = 0;
	if(timeGetTime() >= compareTimeMovement)
	{
		// Game bounds are left y = 100 and right y = 100 from screen edges
		if((GetAsyncKeyState(VK_LEFT) & 0x8000) && (Player.x1 > 100))
		{
			Player.x1 -= 5;
			Player.x2 -= 5;
		}
		else if((GetAsyncKeyState(VK_RIGHT) & 0x8000) && (Player.x2 < 700))
		{
			Player.x1 += 5;
			Player.x2 += 5;
		}
		compareTimeMovement = (timeGetTime() + 1); // sample input only every 5ms
	}

	if(GetAsyncKeyState(VK_F5) & 0x8000 && (timeGetTime() >= lastTimeShow))
	{
		if(Game.showDebug == false) Game.showDebug = true;
		else Game.showDebug = false;
		lastTimeShow = timeGetTime() + 200;
	}
}

void RenderScene()
{
	DDBLTFX ddbltfx;
	memset(&ddbltfx, 0, sizeof(ddbltfx));
	ddbltfx.dwSize = sizeof(ddbltfx);

	Player.Update(); // Fill in Blitting rects;
	Enemy.Update();
	Ball.Update();

	ddbltfx.dwFillColor = Player.color;
	if(FAILED(g_pBack->Blt(&Player.myRect, 0, 0, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx)))
	{
		MessageBox(g_hwnd, "Player Paddle Blt Failed", "Error", MB_OK);
		Log("Player Paddle Blt Failed");
	}

	if(FAILED(g_pBack->Blt(&Enemy.myRect, 0, 0, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx)))
	{
		MessageBox(g_hwnd, "Enemy Paddle Blt Failed", "Error", MB_OK);
		Log("Enemy Paddle Blt Failed");
	}

	ddbltfx.dwFillColor = Ball.color;
	if(FAILED(g_pBack->Blt(&Ball.myRect, 0, 0, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx)))
	{
		MessageBox(g_hwnd, "Ball Blt Failed", "Error", MB_OK);
		Log("Ball Blt Failed");
	}

	RECT SideTop = {98, 143, 702, 145}; // Boarder Rects
	RECT SideRight = {700, 145, 702, 560};
	RECT SideBottom = {98, 560, 702, 562};
	RECT SideLeft = {98, 145, 100, 560};

	ddbltfx.dwFillColor = RGBVAL(20, 20, 20);
	if(FAILED(g_pBack->Blt(&SideTop, 0, 0, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx)))
	{
		MessageBox(g_hwnd, "SideTop blt Failed", "Error", MB_OK);
		Log("SideTop Blt Failed");
	}
	if(FAILED(g_pBack->Blt(&SideRight, 0, 0, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx)))
	{
		MessageBox(g_hwnd, "SideRight blt Failed", "Error", MB_OK);
		Log("SideRight Blt Failed");
	}
	if(FAILED(g_pBack->Blt(&SideBottom, 0, 0, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx)))
	{
		MessageBox(g_hwnd, "SideBottom blt Failed", "Error", MB_OK);
		Log("SideBottom Blt Failed");
	}
	if(FAILED(g_pBack->Blt(&SideLeft, 0, 0, DDBLT_COLORFILL | DDBLT_WAIT, &ddbltfx)))
	{
		MessageBox(g_hwnd, "SideLeft blt Failed", "Error", MB_OK);
		Log("SideLeft Blt Failed");
	}

	RECT destTitleRect;
	destTitleRect.left = 49;
	destTitleRect.top = 1;
	destTitleRect.right = 750;
	destTitleRect.bottom = 122;

	RECT sourceTitleRect;
	sourceTitleRect.left = 0;
	sourceTitleRect.top = 0;
	sourceTitleRect.right = 700;
	sourceTitleRect.bottom = 120;

	memset(&ddbltfx, 0, sizeof(ddbltfx));
	ddbltfx.dwSize = sizeof(ddbltfx);

	if(FAILED(g_pBack->Blt(&destTitleRect, g_pTitleSurface, &sourceTitleRect, DDBLT_WAIT, &ddbltfx)))
	{
		MessageBox(g_hwnd, "Title blt Failed", "Error", MB_OK);
		Log("Title Blt Failed");
	}
}

void Collision()
{
	// DEBUG CODE: - No AI
	//if(Ball.y1 <= Enemy.y2) Ball.Vely = -Ball.Vely;

	if((Ball.y1 <= Enemy.y2) && ((Ball.x2 >= Enemy.x1) && (Ball.x1 <= Enemy.x2)))
		Ball.Vely = -Ball.Vely; // Enemy collide

	if(Ball.x2 >= 700) 
		Ball.Velx = -Ball.Velx; // Right side collide

	if((Ball.y2 >= Player.y1) && ((Ball.x2 >= Player.x1) && (Ball.x1 <= Player.x2)))
	{               
		Ball.Vely = -Ball.Vely; // Player collide
		Game.Score += 100;
		// sndPlaySound("blah.wav", SND_ASYNC); // Stuff this bullshit...
		if(Game.Score >= (500*Game.level)) // 500 points to get to next level
		{
			Ball.Vely--;
			Game.level++;
			for(;;) // Dont choose 0 as Velx
			{
				Ball.Velx = (rand()%4);
				if(Ball.Velx != 0) break;
			}
		}
	}
	if(((Ball.y2 > Player.y1) && (Ball.y1 < Player.y2)) && (((Ball.x2 >= Player.x1) && (Ball.x2 <= Player.x2))  || ((Ball.x1 <= Player.x2) && (Ball.x1 >= Player.x1))))
		Ball.Velx = -Ball.Velx; // Check against paddle sides

	if(Ball.x1 <= 100) 
		Ball.Velx = -Ball.Velx;

	if(Ball.y2 >= 560)
	{
		if(Game.lives == 0) Game.over = true;
		else
		{
			Game.lives--;
			Game.lose = true;
		}
		Ball.x1 = 397;
		Ball.x2 = 402;
		Ball.y1 = 350;
		Ball.y2 = 355;
		Ball.Velx = 0;
		Ball.Vely = 0;
	}

	if(Ball.y1 <= 145)
	{
		Game.win = true;
		Game.lives++;
		Ball.x1 = 397;
		Ball.x2 = 402;
		Ball.y1 = 350;
		Ball.y2 = 355;
		Ball.Velx = 0;
		Ball.Vely = 0;
	}
}

int FPSCounter()
{
	static DWORD oldTime = 0;
	static long FPSnum = 0;
	static long oldFPSnum = 0;

	FPSnum++;

	if(timeGetTime() > oldTime)
	{	
		oldTime = timeGetTime() + 1000;
		oldFPSnum = FPSnum;
		FPSnum = 0;
	}
	return oldFPSnum;
}


